/*
 * Copyright(C) 1999-2022 National Technology & Engineering Solutions
 * of Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
 * NTESS, the U.S. Government retains certain rights in this software.
 *
 * See packages/seacas/LICENSE for details
 */
#pragma once

#define EXODUSII_BUILD_SHARED_LIBS
#if defined(WIN32) || defined(__WIN32__) || defined(_WIN32) || defined(_MSC_VER) ||                \
  defined(__MINGW32__) || defined(_WIN64) || defined(__MINGW64__)
#if defined(EXODUSII_BUILD_SHARED_LIBS)
#  if defined(exoIIc_EXPORTS)
#    define EXODUS_EXPORT __declspec( dllexport ) extern
#  else
#    define EXODUS_EXPORT __declspec( dllimport ) extern
#  endif
#endif
#else
#  define EXODUS_EXPORT extern
#endif
