#ifndef vtklzma_mangle_h
#define vtklzma_mangle_h

#define lzma_alloc vtklzma_lzma_alloc
#define lzma_alloc_zero vtklzma_lzma_alloc_zero
#define lzma_alone_decoder vtklzma_lzma_alone_decoder
#define lzma_alone_decoder_init vtklzma_lzma_alone_decoder_init
#define lzma_alone_encoder vtklzma_lzma_alone_encoder
#define lzma_auto_decoder vtklzma_lzma_auto_decoder
#define lzma_block_buffer_bound vtklzma_lzma_block_buffer_bound
#define lzma_block_buffer_bound64 vtklzma_lzma_block_buffer_bound64
#define lzma_block_buffer_decode vtklzma_lzma_block_buffer_decode
#define lzma_block_buffer_encode vtklzma_lzma_block_buffer_encode
#define lzma_block_compressed_size vtklzma_lzma_block_compressed_size
#define lzma_block_decoder vtklzma_lzma_block_decoder
#define lzma_block_decoder_init vtklzma_lzma_block_decoder_init
#define lzma_block_encoder vtklzma_lzma_block_encoder
#define lzma_block_encoder_init vtklzma_lzma_block_encoder_init
#define lzma_block_header_decode vtklzma_lzma_block_header_decode
#define lzma_block_header_encode vtklzma_lzma_block_header_encode
#define lzma_block_header_size vtklzma_lzma_block_header_size
#define lzma_block_total_size vtklzma_lzma_block_total_size
#define lzma_block_uncomp_encode vtklzma_lzma_block_uncomp_encode
#define lzma_block_unpadded_size vtklzma_lzma_block_unpadded_size
#define lzma_bufcpy vtklzma_lzma_bufcpy
#define lzma_check_finish vtklzma_lzma_check_finish
#define lzma_check_init vtklzma_lzma_check_init
#define lzma_check_is_supported vtklzma_lzma_check_is_supported
#define lzma_check_size vtklzma_lzma_check_size
#define lzma_check_update vtklzma_lzma_check_update
#define lzma_code vtklzma_lzma_code
#define lzma_cputhreads vtklzma_lzma_cputhreads
#define lzma_crc32 vtklzma_lzma_crc32
#define lzma_crc32_table vtklzma_lzma_crc32_table
#define lzma_crc64 vtklzma_lzma_crc64
#define lzma_crc64_table vtklzma_lzma_crc64_table
#define lzma_delta_coder_init vtklzma_lzma_delta_coder_init
#define lzma_delta_coder_memusage vtklzma_lzma_delta_coder_memusage
#define lzma_delta_decoder_init vtklzma_lzma_delta_decoder_init
#define lzma_delta_encoder_init vtklzma_lzma_delta_encoder_init
#define lzma_delta_props_decode vtklzma_lzma_delta_props_decode
#define lzma_delta_props_encode vtklzma_lzma_delta_props_encode
#define lzma_easy_buffer_encode vtklzma_lzma_easy_buffer_encode
#define lzma_easy_decoder_memusage vtklzma_lzma_easy_decoder_memusage
#define lzma_easy_encoder vtklzma_lzma_easy_encoder
#define lzma_easy_encoder_memusage vtklzma_lzma_easy_encoder_memusage
#define lzma_easy_preset vtklzma_lzma_easy_preset
#define lzma_end vtklzma_lzma_end
#define lzma_fastpos vtklzma_lzma_fastpos
#define lzma_filter_decoder_is_supported vtklzma_lzma_filter_decoder_is_supported
#define lzma_filter_encoder_is_supported vtklzma_lzma_filter_encoder_is_supported
#define lzma_filter_flags_decode vtklzma_lzma_filter_flags_decode
#define lzma_filter_flags_encode vtklzma_lzma_filter_flags_encode
#define lzma_filter_flags_size vtklzma_lzma_filter_flags_size
#define lzma_filters_copy vtklzma_lzma_filters_copy
#define lzma_filters_update vtklzma_lzma_filters_update
#define lzma_footer_magic vtklzma_lzma_footer_magic
#define lzma_free vtklzma_lzma_free
#define lzma_get_check vtklzma_lzma_get_check
#define lzma_get_progress vtklzma_lzma_get_progress
#define lzma_header_magic vtklzma_lzma_header_magic
#define lzma_index_append vtklzma_lzma_index_append
#define lzma_index_block_count vtklzma_lzma_index_block_count
#define lzma_index_buffer_decode vtklzma_lzma_index_buffer_decode
#define lzma_index_buffer_encode vtklzma_lzma_index_buffer_encode
#define lzma_index_cat vtklzma_lzma_index_cat
#define lzma_index_checks vtklzma_lzma_index_checks
#define lzma_index_decoder vtklzma_lzma_index_decoder
#define lzma_index_dup vtklzma_lzma_index_dup
#define lzma_index_encoder vtklzma_lzma_index_encoder
#define lzma_index_encoder_init vtklzma_lzma_index_encoder_init
#define lzma_index_end vtklzma_lzma_index_end
#define lzma_index_file_size vtklzma_lzma_index_file_size
#define lzma_index_hash_append vtklzma_lzma_index_hash_append
#define lzma_index_hash_decode vtklzma_lzma_index_hash_decode
#define lzma_index_hash_end vtklzma_lzma_index_hash_end
#define lzma_index_hash_init vtklzma_lzma_index_hash_init
#define lzma_index_hash_size vtklzma_lzma_index_hash_size
#define lzma_index_init vtklzma_lzma_index_init
#define lzma_index_iter_init vtklzma_lzma_index_iter_init
#define lzma_index_iter_locate vtklzma_lzma_index_iter_locate
#define lzma_index_iter_next vtklzma_lzma_index_iter_next
#define lzma_index_iter_rewind vtklzma_lzma_index_iter_rewind
#define lzma_index_memusage vtklzma_lzma_index_memusage
#define lzma_index_memused vtklzma_lzma_index_memused
#define lzma_index_padding_size vtklzma_lzma_index_padding_size
#define lzma_index_prealloc vtklzma_lzma_index_prealloc
#define lzma_index_size vtklzma_lzma_index_size
#define lzma_index_stream_count vtklzma_lzma_index_stream_count
#define lzma_index_stream_flags vtklzma_lzma_index_stream_flags
#define lzma_index_stream_padding vtklzma_lzma_index_stream_padding
#define lzma_index_stream_size vtklzma_lzma_index_stream_size
#define lzma_index_total_size vtklzma_lzma_index_total_size
#define lzma_index_uncompressed_size vtklzma_lzma_index_uncompressed_size
#define lzma_lz_decoder_init vtklzma_lzma_lz_decoder_init
#define lzma_lz_decoder_memusage vtklzma_lzma_lz_decoder_memusage
#define lzma_lz_decoder_uncompressed vtklzma_lzma_lz_decoder_uncompressed
#define lzma_lz_encoder_init vtklzma_lzma_lz_encoder_init
#define lzma_lz_encoder_memusage vtklzma_lzma_lz_encoder_memusage
#define lzma_lzma2_block_size vtklzma_lzma_lzma2_block_size
#define lzma_lzma2_decoder_init vtklzma_lzma_lzma2_decoder_init
#define lzma_lzma2_decoder_memusage vtklzma_lzma_lzma2_decoder_memusage
#define lzma_lzma2_encoder_init vtklzma_lzma_lzma2_encoder_init
#define lzma_lzma2_encoder_memusage vtklzma_lzma_lzma2_encoder_memusage
#define lzma_lzma2_props_decode vtklzma_lzma_lzma2_props_decode
#define lzma_lzma2_props_encode vtklzma_lzma_lzma2_props_encode
#define lzma_lzma_decoder_create vtklzma_lzma_lzma_decoder_create
#define lzma_lzma_decoder_init vtklzma_lzma_lzma_decoder_init
#define lzma_lzma_decoder_memusage vtklzma_lzma_lzma_decoder_memusage
#define lzma_lzma_decoder_memusage_nocheck vtklzma_lzma_lzma_decoder_memusage_nocheck
#define lzma_lzma_encode vtklzma_lzma_lzma_encode
#define lzma_lzma_encoder_create vtklzma_lzma_lzma_encoder_create
#define lzma_lzma_encoder_init vtklzma_lzma_lzma_encoder_init
#define lzma_lzma_encoder_memusage vtklzma_lzma_lzma_encoder_memusage
#define lzma_lzma_encoder_reset vtklzma_lzma_lzma_encoder_reset
#define lzma_lzma_lclppb_decode vtklzma_lzma_lzma_lclppb_decode
#define lzma_lzma_lclppb_encode vtklzma_lzma_lzma_lclppb_encode
#define lzma_lzma_optimum_fast vtklzma_lzma_lzma_optimum_fast
#define lzma_lzma_optimum_normal vtklzma_lzma_lzma_optimum_normal
#define lzma_lzma_preset vtklzma_lzma_lzma_preset
#define lzma_lzma_props_decode vtklzma_lzma_lzma_props_decode
#define lzma_lzma_props_encode vtklzma_lzma_lzma_props_encode
#define lzma_memlimit_get vtklzma_lzma_memlimit_get
#define lzma_memlimit_set vtklzma_lzma_memlimit_set
#define lzma_memusage vtklzma_lzma_memusage
#define lzma_mf_bt2_find vtklzma_lzma_mf_bt2_find
#define lzma_mf_bt2_skip vtklzma_lzma_mf_bt2_skip
#define lzma_mf_bt3_find vtklzma_lzma_mf_bt3_find
#define lzma_mf_bt3_skip vtklzma_lzma_mf_bt3_skip
#define lzma_mf_bt4_find vtklzma_lzma_mf_bt4_find
#define lzma_mf_bt4_skip vtklzma_lzma_mf_bt4_skip
#define lzma_mf_find vtklzma_lzma_mf_find
#define lzma_mf_hc3_find vtklzma_lzma_mf_hc3_find
#define lzma_mf_hc3_skip vtklzma_lzma_mf_hc3_skip
#define lzma_mf_hc4_find vtklzma_lzma_mf_hc4_find
#define lzma_mf_hc4_skip vtklzma_lzma_mf_hc4_skip
#define lzma_mf_is_supported vtklzma_lzma_mf_is_supported
#define lzma_mode_is_supported vtklzma_lzma_mode_is_supported
#define lzma_mt_block_size vtklzma_lzma_mt_block_size
#define lzma_next_end vtklzma_lzma_next_end
#define lzma_next_filter_init vtklzma_lzma_next_filter_init
#define lzma_next_filter_update vtklzma_lzma_next_filter_update
#define lzma_outq_end vtklzma_lzma_outq_end
#define lzma_outq_get_buf vtklzma_lzma_outq_get_buf
#define lzma_outq_init vtklzma_lzma_outq_init
#define lzma_outq_is_readable vtklzma_lzma_outq_is_readable
#define lzma_outq_memusage vtklzma_lzma_outq_memusage
#define lzma_outq_read vtklzma_lzma_outq_read
#define lzma_physmem vtklzma_lzma_physmem
#define lzma_properties_decode vtklzma_lzma_properties_decode
#define lzma_properties_encode vtklzma_lzma_properties_encode
#define lzma_properties_size vtklzma_lzma_properties_size
#define lzma_raw_buffer_decode vtklzma_lzma_raw_buffer_decode
#define lzma_raw_buffer_encode vtklzma_lzma_raw_buffer_encode
#define lzma_raw_coder_init vtklzma_lzma_raw_coder_init
#define lzma_raw_coder_memusage vtklzma_lzma_raw_coder_memusage
#define lzma_raw_decoder vtklzma_lzma_raw_decoder
#define lzma_raw_decoder_init vtklzma_lzma_raw_decoder_init
#define lzma_raw_decoder_memusage vtklzma_lzma_raw_decoder_memusage
#define lzma_raw_encoder vtklzma_lzma_raw_encoder
#define lzma_raw_encoder_init vtklzma_lzma_raw_encoder_init
#define lzma_raw_encoder_memusage vtklzma_lzma_raw_encoder_memusage
#define lzma_rc_prices vtklzma_lzma_rc_prices
#define lzma_sha256_finish vtklzma_lzma_sha256_finish
#define lzma_sha256_init vtklzma_lzma_sha256_init
#define lzma_sha256_update vtklzma_lzma_sha256_update
#define lzma_simple_arm_decoder_init vtklzma_lzma_simple_arm_decoder_init
#define lzma_simple_arm_encoder_init vtklzma_lzma_simple_arm_encoder_init
#define lzma_simple_armthumb_decoder_init vtklzma_lzma_simple_armthumb_decoder_init
#define lzma_simple_armthumb_encoder_init vtklzma_lzma_simple_armthumb_encoder_init
#define lzma_simple_coder_init vtklzma_lzma_simple_coder_init
#define lzma_simple_ia64_decoder_init vtklzma_lzma_simple_ia64_decoder_init
#define lzma_simple_ia64_encoder_init vtklzma_lzma_simple_ia64_encoder_init
#define lzma_simple_powerpc_decoder_init vtklzma_lzma_simple_powerpc_decoder_init
#define lzma_simple_powerpc_encoder_init vtklzma_lzma_simple_powerpc_encoder_init
#define lzma_simple_props_decode vtklzma_lzma_simple_props_decode
#define lzma_simple_props_encode vtklzma_lzma_simple_props_encode
#define lzma_simple_props_size vtklzma_lzma_simple_props_size
#define lzma_simple_sparc_decoder_init vtklzma_lzma_simple_sparc_decoder_init
#define lzma_simple_sparc_encoder_init vtklzma_lzma_simple_sparc_encoder_init
#define lzma_simple_x86_decoder_init vtklzma_lzma_simple_x86_decoder_init
#define lzma_simple_x86_encoder_init vtklzma_lzma_simple_x86_encoder_init
#define lzma_stream_buffer_bound vtklzma_lzma_stream_buffer_bound
#define lzma_stream_buffer_decode vtklzma_lzma_stream_buffer_decode
#define lzma_stream_buffer_encode vtklzma_lzma_stream_buffer_encode
#define lzma_stream_decoder vtklzma_lzma_stream_decoder
#define lzma_stream_decoder_init vtklzma_lzma_stream_decoder_init
#define lzma_stream_encoder vtklzma_lzma_stream_encoder
#define lzma_stream_encoder_mt vtklzma_lzma_stream_encoder_mt
#define lzma_stream_encoder_mt_memusage vtklzma_lzma_stream_encoder_mt_memusage
#define lzma_stream_flags_compare vtklzma_lzma_stream_flags_compare
#define lzma_stream_footer_decode vtklzma_lzma_stream_footer_decode
#define lzma_stream_footer_encode vtklzma_lzma_stream_footer_encode
#define lzma_stream_header_decode vtklzma_lzma_stream_header_decode
#define lzma_stream_header_encode vtklzma_lzma_stream_header_encode
#define lzma_strm_init vtklzma_lzma_strm_init
#define lzma_version_number vtklzma_lzma_version_number
#define lzma_version_string vtklzma_lzma_version_string
#define lzma_vli_decode vtklzma_lzma_vli_decode
#define lzma_vli_encode vtklzma_lzma_vli_encode
#define lzma_vli_size vtklzma_lzma_vli_size
#define vtklzma_tuklib_cpucores vtklzma_vtklzma_tuklib_cpucores
#define vtklzma_tuklib_physmem vtklzma_vtklzma_tuklib_physmem

#endif
