// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
// SPDX-License-Identifier: BSD-3-Clause

/* The TDx support has not been ported, this header defines nothing. */
/* VTK-HeaderTest-Exclude: vtkTDxConfigure.h */
